function plot_confusion(varargin)
%plot_confusion plots the confusion matrix with colorscale, absolute numbers
%   and precision normalized percentages
%
%   USAGE: 
%   plot_confusion(confmat) plots the confmat with integers 1 to n as class labels
%   plot_confusion(targets, outputs) plots the confusion matrix. 
%   Here targets is NxM matrix of true class labels and outputs is LxM matrix of class estimates.
%   plot_confusion(confmat, labels) plots the confmat with the specified labels
%   plot_confusion(targets, outputs, labels) plots the confusion matrix with
%   specified labels.
%
%   Vahe Tshitoyan
%   20/08/2017
%   Modified by Alexei Goumilevski
%   10/18/2018
%
%   Arguments
%   confmat: a square confusion matrix.
%   targets: The target data are ground truth labels in 
%       1-of-N form (in each column, a single element is 1 to indicate the 
%       correct class, and all other elements are 0). 
%   outputs: The output data are the 
%       outputs from a neural network that performs classification. They can 
%       either be in 1-of-L form, or may also be probabilities where each column 
%       sums to 1.
%   labels (optional):  vector of class labels.
%   flip (optional): 

% number of arguments

flip = false;
title_matrix = 'Confusion Matrix';
labels = {};
for i=1:nargin
    if isa(varargin{i},'logical')
        flip = varargin{i};
    elseif isa(varargin{i},'char')
        title_matrix = varargin{i};
    elseif iscell(varargin{i})
        labels = varargin{i};
    end
end
if isempty(labels)
    if size(varargin{1},1) == 2
        labels = {'True','False'};
    else
        labels = 1:size(varargin{1},1);
        labels = strsplit(num2str(labels));
    end
end
if flip
    labels = fliplr(labels);
end
labels{1+end} = '';
    
switch (nargin)
    case 0
       confmat = 1;
    case 1
       confmat = varargin{1};
    otherwise
        if ismatrix(varargin{1}) && ismatrix(varargin{2})
            targets = varargin{1};
            outputs = varargin{2};
            % Unknown/dont-care targets
            known = find(isfinite(sum(targets,1)));
            targets = targets(:,known);
            outputs = outputs(:,known);
            % Build confusion matrix
            confmat = get_confusion_matrix(targets,outputs,flip);
        else
            confmat = varargin{1};
        end
end

confmat(isnan(confmat))=0; % in case there are NaN elements
numColumns = 1+size(confmat, 1); % number of labels
temp = zeros(numColumns,numColumns);
temp(1:numColumns-1,1:numColumns-1) = confmat;
confmat = temp;

% Calculate the percentage accuracies
confpercent = 100*confmat./sum(sum(confmat));
%accuracy = 100*trace(confmat)/sum(confmat(:));

% Calculate statistics
for i=1:numColumns
    confmat(i,numColumns) = 100*confmat(i,i)/sum(confmat(i,:));
    confpercent(i,numColumns) = 100 - confmat(i,numColumns);
    confmat(numColumns,i) = 100*confmat(i,i)/sum(confmat(:,i));
    confpercent(numColumns,i) = 100 - confmat(numColumns,i);
end
confmat(numColumns,numColumns) = 100*sum(diag(confmat))/sum(sum(confmat(1:numColumns-1,1:numColumns-1)));
confpercent(numColumns,numColumns) = 100 - confmat(numColumns,numColumns);

n = numColumns*(numColumns-1);

% Plotting the colors
img = ones(numColumns,numColumns);
img(1:1+numColumns:end) = 0.7;
img(numColumns,:) = 0.5;
img(:,numColumns) = 0.5;
img(numColumns,numColumns) = 0.2;
imagesc(img);
title(title_matrix);
ylabel('Output Class'); 
xlabel('Target Class');

% Plot lines
hold on
lw = 1;
for i=0:numColumns-1
    if i==numColumns-1
        lw = 2;
    end
    plot([0.5+i,0.5+i],[0,1+numColumns],'k','LineWidth',lw)
    plot([0,1+numColumns],[0.5+i,0.5+i],'k','LineWidth',lw)
end
plot([0.5+numColumns,0.5+numColumns],[0,1+numColumns],'k','LineWidth',lw)
plot([0,1+numColumns],[0.5+numColumns,0.5+numColumns],'k','LineWidth',lw)

% Set the colormap
colormap([0.7 0.7 0.7;0.85 0.85 0.85;0.5 1 0.5;1 0.5 0.5]);

% Create strings from the matrix values and remove spaces
if numColumns-1 == 2
    if flip
        attr = {'True Negatives','False Negatives',''; 'False Positives','True Positives','';'','',''};
    else
        attr = {'True Positives','False Positives',''; 'False Negatives','True Negatives','';'','',''};
    end
    textStrings = num2str([confmat(:), confpercent(:)], '%d\n%-.1f%%\n');
    textStrings = [textStrings, char(attr)];
    textStrings = strtrim(cellstr(textStrings));
    temp = {};
    if flip
        temp{1} = {num2str(confmat(1,numColumns),'%.1f%%\n'),num2str(confpercent(1,numColumns),'%.1f%%'),''};
        temp{2} = {num2str(confmat(2,numColumns),'%.1f%%\n'),num2str(confpercent(2,numColumns),'%.1f%%'),'Precision'};
    else
        temp{1} = {num2str(confmat(1,numColumns),'%.1f%%\n'),num2str(confpercent(1,numColumns),'%.1f%%'),'Precision'};
        temp{2} = {num2str(confmat(2,numColumns),'%.1f%%\n'),num2str(confpercent(2,numColumns),'%.1f%%'),''};
    end
    temp{3} =  {num2str(confmat(3,numColumns),'%.1f%%\n'),num2str(confpercent(3,numColumns),'%.1f%%'),'Accuracy'};
    textStrings(n+1:end) = strtrim(temp);
    temp = {};
    if flip
        temp{1} = {num2str(confmat(numColumns,1),'%.1f%%\n'), num2str(confpercent(numColumns,1),'%.1f%%\n'),'Specificity'};
        temp{2} = {num2str(confmat(numColumns,2),'%.1f%%\n'), num2str(confpercent(numColumns,2),'%.1f%%\n'),'Sensitivity (Recall)'};
    else
        temp{1} = {num2str(confmat(numColumns,1),'%.1f%%\n'), num2str(confpercent(numColumns,1),'%.1f%%\n'),'Sensitivity (Recall)'};
        temp{2} = {num2str(confmat(numColumns,2),'%.1f%%\n'), num2str(confpercent(numColumns,2),'%.1f%%\n'),'Specificity'};
    end
    textStrings(numColumns:numColumns:end-1) = strtrim(temp);
else
    textStrings = num2str([confmat(:), confpercent(:)], '%d\n%-.1f%%\n');
    textStrings = strtrim(cellstr(textStrings));
    temp = num2str([confmat(:,numColumns), confpercent(:,numColumns)], '%.1f%%\n');
    textStrings(n+1:end) = strtrim(cellstr(temp));
    temp = num2str([confmat(numColumns,1:end-1)', confpercent(numColumns,1:end-1)'], '%.1f%%\n');
    textStrings(numColumns:numColumns:end-1) = strtrim(cellstr(temp));
end


% Create x and y coordinates for the strings and plot them
[x,y] = meshgrid(1:numColumns);
text(x(:),y(:),textStrings(:),'HorizontalAlignment','center','FontWeight','bold');

% Setting the axis labels
set(gca,'XTick',1:numColumns,...
    'XTickLabel',labels,...
    'YTick',1:numColumns,...
    'YTickLabel',labels,...
    'TickLength',[0 0]);
hold off
end